/*
LC.PRC
This is a Gauss procedure which calculates the LC test
for parameter instability in linear models as reported
in Hansen (1992) Journal of Policy Modeling.

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/


The format for the command is

lc(dat,datname)

The matrix "dat" is a nx(k+1) matrix, with the dependent
(y) variable in the first colume, and the independent (x)
variables in the remaining columes.  If a constant is desired
in the regression, it should be included in the dat matrix.
The vector "datname" is a (k+1)x1 list of variable names,
for the columes of "dat".

The output is printed to the screen.

*/


proc (0) = lc(dat,datname);
local n,k,y,x,m,b,e,xe,e2,sig2,e2m,se,datnames,
f,s,v,lj,li,bb,ff,t1,pr,r2;

n=rows(dat);
k=cols(dat)-1;
y=dat[.,1];
x=dat[.,2:k+1];
m=invpd(moment(x,0));
b=m*(x'y);
e=y-x*b;
xe=x.*e;
e2=e.^2;
sig2=meanc(e2);
e2m=e2-sig2;
se=sqrt(diag(m*moment(xe,0)*m))|sqrt(meanc(e2m.^2)/n);
r2=1-sumc(e2)/sumc((y-meanc(y)).^2);
f=xe~e2m;
s=cumsumc(f);
v=f'f;
lj=sumc(sumc(s.*(s*invpd(v))))/n;
li=sumc(s.^2)./diag(v)/n;
datnames=datname[2:k+1]|"Variance";
bb=datnames~(b|sig2)~se~li;
ff = "#*.*lG";
t1="" $+ "Dependent Variable = " $+ datname[1];
$t1;
"";
pr = printfm("Variable"~"Estimate"~"St Error"~"Lc",0~0~0~0,ff~16~8);"";
"       ____________________________________________________";
pr = printfm(bb,0~1~1~1,ff~16~8);"";
"       ____________________________________________________";
"";"";
"Joint LC      " lj;
"R-squared   " r2;
"";"";
"************************************************************";
"";"";
endp;
